/* Datei:  comir_tmr.c
 * Autor: gkemnitz
 * letzte nderung: 27.09.2016
 *
 * Quellprogramm fr den Gertetreiber comir_tmr
 */ 

#include "comir_tmr.h"
#include <avr/interrupt.h>

#define WGM_CTC	  0b0100    // Clear Timer on Compare
#define CS256     0b100     // Vorteiler 256

uint32_t tmr_ct;            // Aufwrtszhler Programmzeit
uint16_t tmr_array[4];      // Abwrtszhler Wartezeit

// Timer 1 fr OCRA-Interrupts alle 10 ms initialisieren
void tmr_init(){ 
  TCCR1A = WGM_CTC & 0b11;  // Betriebsart & Zhltakt
  TCCR1B = (WGM_CTC & 0b1100)<<1 | (CS256 & 0b111);
  OCR1A  = 3125;            // Vergleichswert fr 100 ms
  tmr_ct = 0;               // Laufzeitzhler lschen
  TIMSK1 |= 1<<OCIE1A;	    // Vergleichs-Interrupt A ein
}


ISR(TIMER1_COMPA_vect){     // Vergleichs-Interrupt
  uint8_t idx;              // alle 100 ms
  tmr_ct++;                 // Programmzeit zhlen
  for (idx=0; idx<4; idx++) // fr alle 4 Wartezhler
    if (tmr_array[idx])     // wenn ungleich null
      tmr_array[idx]--;     // abwrts zhlen
}

void tmr_start(uint16_t tw, uint8_t nr){
  uint8_t  tmp = TIMSK1;    // Int.-Zustand sichern
  TIMSK1 &= ~(1<<OCIE1A);	// Vergleichs-Interrupt A aus
  tmr_array[nr & 0b11] = tw;// Wartezeit schreiben
  TIMSK1 = tmp;             // Int.-Zustand wiederherst.
}

uint16_t tmr_restzeit(uint8_t nr){ // Restzeit lesen
  uint8_t  tmp = TIMSK1;    // Int.-Zustand sichern
  TIMSK1 &= ~(1<<OCIE1A);	// Vergleichs-Interrupt A aus
  uint16_t z = tmr_array[nr & 0b11]; // Restzeit lesen
  TIMSK1 = tmp;             // Int.-Zustand wiederherst.
	return z;
}

uint32_t tmr_get(){         // Zeitzaehler lesen
  uint8_t  tmp = TIMSK1;    // Int.-Zustand sichern
  TIMSK1 &= ~(1<<OCIE1A);	// Vergleichs-Interrupt A aus
  uint32_t z = tmr_ct;      // Uhrzeit lesen
  TIMSK1 = tmp;             // Int.-Zustand wiederherst.
	return z;
}
